@@ -1,11 +1,11 @@
-(defproject puppetlabs/kitchensink "3.2.1"
+(defproject puppetlabs/kitchensink "3.2.5"
   :description "Clojure utility functions"
   :license {:name "Apache License, Version 2.0"
             :url "http://www.apache.org/licenses/LICENSE-2.0.html"}
 
   :min-lein-version "2.9.1"
 
-  :parent-project {:coords [puppetlabs/clj-parent "4.6.17"]
+  :parent-project {:coords [puppetlabs/clj-parent "5.6.6"]
                    :inherit [:managed-dependencies]}
 
   ;; Abort when version ranges or version conflicts are detected in
@@ -17,6 +17,7 @@
                  [org.clojure/tools.logging]
                  [org.clojure/tools.cli]
 
+                 [org.apache.commons/commons-compress]
                  [clj-time]
                  [clj-commons/fs]
                  [slingshot]
@@ -36,8 +37,20 @@
 
   ;; this plugin is used by jenkins jobs to interrogate the project version
   :plugins [[lein-project-version "0.1.0"]
+            [jonase/eastwood "1.2.2" :exclusions [org.clojure/clojure]]
             [lein-parent "0.3.7"]]
 
+  :eastwood {:ignored-faults {:unused-ret-vals {puppetlabs.kitchensink.classpath [{:line 93}]}
+                              :deprecations {puppetlabs.kitchensink.classpath [{:line 66}
+                                                                               {:line 91}]
+                                             puppetlabs.kitchensink.core true
+                                             puppetlabs.kitchensink.core-test true}
+                              :reflection {puppetlabs.kitchensink.file [{:line 62}]
+                                           puppetlabs.kitchensink.core [{:line 929}]}
+                              :constant-test {puppetlabs.kitchensink.core-test [{:line 726}
+                                                                                {:line 733}]}}
+             :continue-on-exception true}
+
   :test-selectors {:default (complement :slow)
                    :slow :slow}
 
