@@ -15,12 +15,15 @@
             [clojure.string :as string]
             [clojure.tools.cli :as cli]
             [clojure.tools.logging :as log]
-            digest
+            [digest]
             [me.raynes.fs :as fs]
             [slingshot.slingshot :refer [throw+]])
-  (:import [java.io File Reader StringWriter]
+  (:import (java.io File Reader StringWriter)
+           (java.time ZoneId ZoneOffset ZonedDateTime)
+           (java.time.format DateTimeFormatter)
            javax.naming.ldap.LdapName
-           [org.ini4j BasicProfileSection Config Ini]))
+           (javax.naming.ldap Rdn)
+           (org.ini4j BasicProfileSection Config Ini)))
 
 (defn error-map
   [kind message]
@@ -205,11 +208,11 @@ to be a zipper."
   "Performs division on the supplied arguments, substituting `default`
   when the divisor is 0"
   ([dividend divisor]
-    (quotient dividend divisor 0))
+   (quotient dividend divisor 0))
   ([dividend divisor default]
-    (if (zero? divisor)
-      default
-      (/ dividend divisor))))
+   (if (zero? divisor)
+     default
+     (/ dividend divisor))))
 
 ;; ## Numerics
 
@@ -220,7 +223,7 @@ to be a zipper."
   {:pre  [(string? s)]
    :post [(or (integer? %) (nil? %))]}
   (try (Integer/parseInt s)
-    (catch java.lang.NumberFormatException e
+    (catch NumberFormatException _e
       nil)))
 
 (defn parse-float
@@ -230,7 +233,7 @@ to be a zipper."
   {:pre  [(string? s)]
    :post [(or (float? %) (nil? %))]}
   (try (Float/parseFloat s)
-    (catch java.lang.NumberFormatException e
+    (catch NumberFormatException _e
       nil)))
 
 (defn parse-number
@@ -317,12 +320,12 @@ to be a zipper."
   already. Returns a list by default, or you can use a constructor func
   as the second arg."
   ([item]
-    (as-collection item list))
+   (as-collection item list))
   ([item constructor]
-    {:post [(coll? %)]}
-    (if (coll? item)
-      item
-      (constructor item))))
+   {:post [(coll? %)]}
+   (if (coll? item)
+     item
+     (constructor item))))
 
 (defn seq-contains?
   "True if seq contains elm"
@@ -345,13 +348,13 @@ to be a zipper."
   "If coll `contains?` any of the keys in ks, returns the first such
   key.  Otherwise returns nil."
   [coll ks]
-  (some #(if (contains? coll %) %) ks))
+  (some #(when (contains? coll %) %) ks))
 
 (defn excludes-some
   "If coll `excludes?` any of the keys in ks, returns the first such
   key.  Otherwise returns nil."
   [coll ks]
-  (some #(if (excludes? coll %) %) ks))
+  (some #(when (excludes? coll %) %) ks))
 
 (defn mapvals
   "Return map `m`, with each value transformed by function `f`.
@@ -359,13 +362,13 @@ to be a zipper."
   You may also provide an optional list of keys `ks`; if provided, only the
   specified keys will be modified."
   ([f m]
-    (into {} (for [[k v] m] [k (f v)])))
+   (into {} (for [[k v] m] [k (f v)])))
   ([f ks m]
     ;; would prefer to share code between the two implementations here, but
     ;; the `into` is much faster for the base case and the reduce is much
     ;; faster for any case where we're operating on a subset of the keys.
     ;; It seems like `select-keys` is fairly expensive.
-    (reduce (fn [m k] (update-in m [k] f)) m ks)))
+   (reduce (fn [m k] (update-in m [k] f)) m ks)))
 
 (defn mapkeys
   "Return map `m`, with each key transformed by function `f`"
@@ -392,13 +395,13 @@ to be a zipper."
   returns a modified map with the specified key removed.  If the value is not `nil`,
   simply returns the original map."
   ([m k]
-    {:pre  [(map? m)]
-     :post [(map? %)]}
-    (if (nil? (m k))
+   {:pre  [(map? m)]
+    :post [(map? %)]}
+   (if (nil? (m k))
       (dissoc m k)
       m))
   ([m k & ks]
-    (let [ret (dissoc-if-nil m k)]
+   (let [ret (dissoc-if-nil m k)]
       (if ks
         (recur ret (first ks) (next ks))
         ret))))
@@ -608,9 +611,9 @@ to be a zipper."
   "Returns a timestamp string for the given `time`, or the current time if none
   is provided. The format of the timestamp is eg. 2012-02-23T22:01:39.539Z."
   ([]
-    (timestamp (time/now)))
+   (timestamp (time/now)))
   ([time]
-    (time-format/unparse (time-format/formatters :date-time) time)))
+   (time-format/unparse (time-format/formatters :date-time) time)))
 
 ;; ## Exception handling
 
@@ -627,10 +630,11 @@ to be a zipper."
   "Executes the supplied fn repeatedly. Execution may be stopped with an
   InterruptedException."
   [f on-error]
-  (if (try
+  (when
+    (try
         (f)
         true
-        (catch InterruptedException e
+        (catch InterruptedException _e
           false)
         (catch Throwable e
           (on-error e)
@@ -712,7 +716,6 @@ to be a zipper."
   You may also call with no arguments, in which case the prefix string will be
   empty."
   [& args]
-  temp-dir
   (if (empty? args)
     (delete-on-exit (fs/temp-dir nil))
     (delete-on-exit (apply fs/temp-dir args))))
@@ -784,12 +787,12 @@ to be a zipper."
   directory, we return a merged version of parsing all the .ini files
   in the directory (we do not do a recursive find of .ini files)."
   ([path]
-    (inis-to-map path "*.ini"))
+   (inis-to-map path "*.ini"))
   ([path glob-pattern]
-    {:pre  [(or (string? path)
+   {:pre  [(or (string? path)
               (instance? java.io.File path))]
-     :post [(map? %)]}
-    (let [files (if-not (fs/directory? path)
+    :post [(map? %)]}
+   (let [files (if-not (fs/directory? path)
                   [path]
                   (fs/glob (fs/file path glob-pattern)))]
       (->> files
@@ -868,8 +871,8 @@ to be a zipper."
     still invalid in some way."
   ([args specs] (cli! args specs nil))
   ([args specs required-args]
-  (let [specs (conj specs ["-h" "--help" "Show help" :default false])
-        {:keys [options arguments summary errors]} (cli/parse-opts args specs)]
+   (let [specs (conj specs ["-h" "--help" "Show help" :default false])
+         {:keys [options arguments summary errors]} (cli/parse-opts args specs)]
     (when errors
       (let [msg (str
                   "\n\n"
@@ -880,7 +883,7 @@ to be a zipper."
         (throw+ (error-map ::cli-error msg))))
     (when (:help options)
       (throw+ (error-map ::cli-help summary)))
-    (when-let [missing-field (some #(if (not (contains? options %)) %) required-args)]
+    (when-let [missing-field (some #(when-not (contains? options %) %) required-args)]
       (let [msg (str
                   "\n\n"
                   (format "Missing required argument '--%s'!" (name missing-field))
@@ -897,8 +900,8 @@ to be a zipper."
 ;; These functions are only used by PuppetDB and they should likely move back into that
 ;; project until they can be refactored away over functions from the jvm-ca library.
 
-(defn cn-for-dn
-  "Deprecated. Use functions from `jvm-certificate-authority`.
+(defn ^:deprecated cn-for-dn
+  "Deprecated. Use functions from https://github.com/puppetlabs/jvm-ssl-utils instead.
 
   Extracts the CN (common name) from an LDAP DN (distinguished name).
 
@@ -921,13 +924,13 @@ to be a zipper."
   (some->> dn
     (LdapName.)
     (.getRdns)
-    (filter #(= "CN" (.getType %)))
+    (filter #(= "CN" (.getType ^Rdn %)))
     (first)
     (.getValue)
     (str)))
 
-(defn cn-for-cert
-  "Deprecated. Use functions from `jvm-certificate-authority`.
+(defn ^:deprecated cn-for-cert
+  "Deprecated. Use functions from https://github.com/puppetlabs/jvm-ssl-utils instead.
 
   Extract the CN from the DN of an x509 certificate. See `cn-for-dn` for details
   on how extraction is performed.
@@ -935,7 +938,7 @@ to be a zipper."
   If no CN exists in the certificate DN, nil is returned."
   [^java.security.cert.X509Certificate cert]
   (-> cert
-    (.getSubjectDN)
+    (.getSubjectX500Principal)
     (.getName)
     (cn-for-dn)))
 
@@ -955,7 +958,7 @@ to be a zipper."
             (instance? java.io.File whitelist))]
    :post [(fn? %)]}
   (let [allowed? (set (lines whitelist))]
-    (fn [{:keys [ssl-client-cn scheme] :as req}]
+    (fn [{:keys [ssl-client-cn scheme]}]
       (or (= scheme :http)
         (allowed? ssl-client-cn)))))
 
@@ -1024,7 +1027,7 @@ to be a zipper."
   (try
     (java.util.UUID/fromString uuid)
     true
-    (catch IllegalArgumentException e
+    (catch IllegalArgumentException _e
       false)))
 
 ;; ## System interface
@@ -1057,6 +1060,11 @@ to be a zipper."
   "Returns a string of the currently running java version"
   (System/getProperty "java.version"))
 
+(defn get-lein-project-version
+  "Get the version of the specified leinigen project from the runtime."
+  [project]
+  (System/getProperty (format "%s.version" project)))
+
 ;; control flow
 
 (defmacro cond-let
@@ -1180,3 +1188,86 @@ to be a zipper."
   (let [token #"[a-zA-Z0-9!#$%&'*+.^_`|~-]+"
         matcher (format "^(%s/%s)(?:[ \t;]|$)" token token)]
     (second (re-find (re-pattern matcher) content-type))))
+
+(defn ZonedDateTime->timestamp-string
+  ^String [^ZonedDateTime zdt]
+  (.format DateTimeFormatter/ISO_OFFSET_DATE_TIME zdt))
+
+(defn now->timestamp-string
+  "Using the java native libraries, using the system clock, create a UTC based iso8601 timestamp"
+  ^String  []
+  (ZonedDateTime->timestamp-string (ZonedDateTime/now ZoneOffset/UTC)))
+
+(defn timestamp-string->ZonedDateTime
+  "Given an iso8601 timestamp with offset, convert it into a java.time.ZonedDateTime.
+  Throws DateTimeParseException if string can't be parsed"
+  ^ZonedDateTime [^String timestamp]
+  (ZonedDateTime/from (.parse DateTimeFormatter/ISO_OFFSET_DATE_TIME timestamp)))
+
+(defn ZonedDateTime->utc-ZonedDateTime
+  "given a zoned date time, convert it to the utc timezone"
+  ^ZonedDateTime [^ZonedDateTime zdt]
+  (.withZoneSameInstant zdt (ZoneId/of "UTC")))
+
+(defn is-only-number?
+  "Given a string, does the string only consist of numeric characters (0 through 9)"
+  [a]
+  (some? (re-find #"^\d+$" a)))
+
+(defn starts-with-zero?
+  "Given a string, does it start with zero?"
+  [a]
+  (some? (re-find #"^0" a)))
+
+(defn compare-versions
+  "Similar in concept to clojure's `compare` but for version strings.  Based on puppet's versioncmp function
+  https://github.com/puppetlabs/puppet/blob/9ab8526d0bc5f0fe4e29e7dac9832307d8af2e48/lib/puppet/util/package.rb#L3 but does
+  not ignore trailing zeros.
+
+  Given two version strings, do a logical comparison of the two.  If the first is an earlier version that the second,
+  return a positive number.  If they are equivalent versions, return 0, if the first is newer than the second, return a
+  negative number."
+  [a b]
+  (let [version-regular-expression #"[\-.]|\d+|[^\-.\d]+"
+        a-matches (re-seq version-regular-expression a)
+        b-matches (re-seq version-regular-expression b)]
+    (if (and (pos? (count a-matches))
+             (pos? (count b-matches)))
+      (loop [current-a-matches a-matches
+             current-b-matches b-matches]
+        (let [current-a (first current-a-matches)
+              current-b (first current-b-matches)]
+          (cond
+            (and (nil? current-a) (nil? current-b))
+            0
+
+            (nil? current-a)
+            (compare a b)
+
+            (nil? current-b)
+            (compare a b)
+
+            (= current-a current-b)
+            (recur (rest current-a-matches) (rest current-b-matches))
+
+            (= "-" current-a)
+            -1
+
+            (= "-" current-b)
+            1
+
+            (= "." current-a)
+            -1
+
+            (= "." current-b)
+            1
+
+            (and (is-only-number? current-a) (is-only-number? current-b))
+            (if (or (starts-with-zero? current-a)
+                    (starts-with-zero? current-b))
+              (compare (.toUpperCase ^String current-a) (.toUpperCase ^String current-b))
+              (compare (parse-int current-a) (parse-int current-b)))
+
+            :else
+            (compare (.toUpperCase ^String current-a) (.toUpperCase ^String current-b)))))
+      (compare a b))))
\ No newline at end of file
